@@ -2,6 +2,7 @@ import * as symbol from './symbol.js'
 import * as time from './time.js'
 import * as env from './environment.js'
 import * as func from './function.js'
+import * as json from './json.js'
 
 export const BOLD = symbol.create()
 export const UNBOLD = symbol.create()
@@ -15,17 +16,22 @@ export const UNCOLOR = symbol.create()
 
 /* c8 ignore start */
 /**
- * @param {Array<string|Symbol|Object|number>} args
+ * @param {Array<undefined|string|Symbol|Object|number|function():any>} args
  * @return {Array<string|object|number>}
  */
 export const computeNoColorLoggingArgs = args => {
+  if (args.length === 1 && args[0]?.constructor === Function) {
+    args = /** @type {Array<string|Symbol|Object|number>} */ (/** @type {[function]} */ (args)[0]())
+  }
   const strBuilder = []
   const logArgs = []
   // try with formatting until we find something unsupported
   let i = 0
   for (; i < args.length; i++) {
     const arg = args[i]
-    if (arg.constructor === String || arg.constructor === Number) {
+    if (arg === undefined) {
+      strBuilder.push('undefined')
+    } else if (arg.constructor === String || arg.constructor === Number) {
       strBuilder.push(arg)
     } else if (arg.constructor === Object) {
       logArgs.push(JSON.stringify(arg))
@@ -56,6 +62,9 @@ export const createModuleLogger = (_print, moduleName) => {
   return !doLogging
     ? func.nop
     : (...args) => {
+        if (args.length === 1 && args[0]?.constructor === Function) {
+          args = args[0]()
+        }
         const timeNow = time.getUnixTime()
         const timeDiff = timeNow - lastLoggingTime
         lastLoggingTime = timeNow
@@ -63,11 +72,20 @@ export const createModuleLogger = (_print, moduleName) => {
           color,
           moduleName,
           UNCOLOR,
-          ...args.map((arg) =>
-            (typeof arg === 'string' || typeof arg === 'symbol')
-              ? arg
-              : JSON.stringify(arg)
-          ),
+          ...args.map((arg) => {
+            if (arg != null && arg.constructor === Uint8Array) {
+              arg = Array.from(arg)
+            }
+            const t = typeof arg
+            switch (t) {
+              case 'string':
+              case 'symbol':
+                return arg
+              default: {
+                return json.stringify(arg)
+              }
+            }
+          }),
           color,
           ' +' + timeDiff + 'ms'
         )
